@@ -43,6 +43,7 @@
 
 #![warn(missing_docs)]
 
+use std::{ptr, str};
 use std::any::Any;
 use std::collections::BTreeMap;
 use std::ffi::CStr;
@@ -50,7 +51,7 @@ use std::ffi::CString;
 use std::panic::catch_unwind;
 use std::path::PathBuf;
 use std::ptr::null_mut;
-use std::str;
+use std::str::from_utf8;
 
 use bytes::Bytes;
 use chrono::Local;
@@ -64,12 +65,12 @@ use rand::prelude::*;
 use serde_json::json;
 use uuid::Uuid;
 
+use pact_matching::logging::fetch_buffer_contents;
 use pact_matching::models::{HttpPart, Pact, RequestResponseInteraction};
 use pact_matching::models::generators::Generators;
 use pact_matching::models::matchingrules::{MatchingRule, RuleLogic};
 use pact_matching::models::matchingrules::MatchingRules;
 use pact_matching::models::message::Message;
-use pact_matching::time_utils::{parse_pattern, to_chrono_pattern};
 use pact_mock_server::{MANAGER, MockServerError, tls::TlsConfigBuilder, WritePactFileErr};
 use pact_mock_server::server_manager::ServerManager;
 use pact_models::bodies::OptionalBody::{Null, Present};
@@ -77,6 +78,7 @@ use pact_models::bodies::OptionalBody;
 use pact_models::content_types::ContentType;
 use pact_models::PactSpecification;
 use pact_models::provider_states::ProviderState;
+use pact_models::time_utils::{parse_pattern, to_chrono_pattern};
 
 use crate::bodies::{empty_multipart_body, file_as_multipart_body, MultipartBody, process_json, request_multipart, response_multipart};
 use crate::bodies::process_object;
@@ -87,7 +89,7 @@ pub use pact_matching_ffi::log::{
   logger_apply,
   logger_attach_sink,
   logger_init,
-  fetch_memory_buffer,
+  fetch_log_buffer,
   log_to_stdout,
   log_to_stderr,
   log_to_file,
@@ -438,6 +440,47 @@ pub extern fn write_pact_file(mock_server_port: i32, directory: *const c_char, o
   }
 }
 
+
+/// Fetch the logs for the mock server. This needs the memory buffer log sink to be setup before
+/// the mock server is started. Returned string will be freed with the `cleanup_mock_server`
+/// function call.
+///
+/// Will return a NULL pointer if the logs for the mock server can not be retrieved.
+#[no_mangle]
+pub extern fn mock_server_logs(mock_server_port: i32) -> *const c_char {
+  let result = catch_unwind(|| {
+    MANAGER.lock().unwrap()
+      .get_or_insert_with(ServerManager::new)
+      .find_mock_server_by_port_mut(mock_server_port as u16, &|mock_server| {
+        match from_utf8(&fetch_buffer_contents(&mock_server.id)) {
+          Ok(contents) => match CString::new(contents.to_string()) {
+            Ok(c_str) => {
+              let p = c_str.as_ptr();
+              mock_server.resources.push(c_str);
+              p
+            },
+            Err(err) => {
+              error!("Failed to copy in-memory log buffer - {}", err);
+              ptr::null()
+            }
+          }
+          Err(err) => {
+            error!("Failed to convert in-memory log buffer to UTF-8 = {}", err);
+            ptr::null()
+          }
+        }
+      })
+  });
+
+  match result {
+    Ok(val) => val.unwrap_or_else(|| ptr::null()),
+    Err(cause) => {
+      error!("Caught a general panic: {:?}", cause);
+      ptr::null()
+    }
+  }
+}
+
 /// Creates a new Pact model and returns a handle to it.
 ///
 /// * `consumer_name` - The name of the consumer for the pact.
@@ -612,7 +655,7 @@ pub extern fn with_query_parameter(
 ///
 /// For non-body values (headers, query, path etc.) extract out the value from any matchers
 /// and apply the matchers/generators to the model
-fn from_integration_json(rules: &mut MatchingRules, generators: &mut Generators, value: &String, path: &String, category: &str) -> String {
+fn from_integration_json(rules: &mut MatchingRules, generators: &mut Generators, value: &str, path: &str, category: &str) -> String {
   let category = rules.add_category(category);
 
   match serde_json::from_str(&value) {
@@ -660,7 +703,7 @@ pub extern fn with_pact_metadata(
     let name = convert_cstr("name", name).unwrap_or_default();
     let value = convert_cstr("value", value).unwrap_or_default();
 
-    if namespace != "" {
+    if !namespace.is_empty() {
       let mut child = BTreeMap::new();
       child.insert(name.to_string(), value.to_string());
       inner.pact.metadata.insert(namespace.to_string(), child);
@@ -695,8 +738,16 @@ pub extern fn with_header(
       };
 
       let value = match part {
-        InteractionPart::Request => from_integration_json(&mut inner.request.matching_rules, &mut inner.request.generators, &value.to_string(), &format!("{}", &name).to_string(), "header"),
-        InteractionPart::Response => from_integration_json(&mut inner.response.matching_rules, &mut inner.response.generators, &value.to_string(), &format!("{}", &name).to_string(), "header")
+        InteractionPart::Request => from_integration_json(&mut inner.request.matching_rules,
+                                                          &mut inner.request.generators,
+                                                          &value.to_string(),
+                                                          &name.to_string(),
+                                                          "header"),
+        InteractionPart::Response => from_integration_json(&mut inner.response.matching_rules,
+                                                           &mut inner.response.generators,
+                                                           &value.to_string(),
+                                                           &name.to_string(),
+                                                           "header")
       };
 
       let updated_headers = headers.map(|mut h| {
@@ -757,7 +808,7 @@ pub extern fn with_body(
   content_type: *const c_char,
   body: *const c_char
 ) -> bool {
-  let content_type = convert_cstr("content_type", content_type).unwrap_or_else(|| "text/plain");
+  let content_type = convert_cstr("content_type", content_type).unwrap_or("text/plain");
   let body = convert_cstr("body", body).unwrap_or_default();
   let content_type_header = "Content-Type".to_string();
   interaction.with_interaction(&|_, mock_server_started, inner| {
@@ -1047,7 +1098,7 @@ pub extern fn with_multipart_file(
   file: *const c_char,
   part_name: *const c_char
 ) -> StringResult {
-  let part_name = convert_cstr("part_name", part_name).unwrap_or_else(|| "file");
+  let part_name = convert_cstr("part_name", part_name).unwrap_or("file");
   match convert_cstr("content_type", content_type) {
     Some(content_type) => {
       match interaction.with_interaction(&|_, mock_server_started, inner| {
@@ -1058,7 +1109,7 @@ pub extern fn with_multipart_file(
               InteractionPart::Response => response_multipart(&mut inner.response, &body.boundary, body.body, &content_type, part_name)
             };
             if mock_server_started {
-              Err(format!("with_multipart_file: This Pact can not be modified, as the mock server has already started"))
+              Err("with_multipart_file: This Pact can not be modified, as the mock server has already started".to_string())
             } else {
               Ok(())
             }
@@ -1121,8 +1172,8 @@ fn convert_ptr_to_mime_part_body(file: *const c_char, part_name: &str) -> Result
 /// Returns a new `MessagePactHandle`.
 #[no_mangle]
 pub extern fn new_message_pact(consumer_name: *const c_char, provider_name: *const c_char) -> handles::MessagePactHandle {
-  let consumer = convert_cstr("consumer_name", consumer_name).unwrap_or_else(|| "Consumer");
-  let provider = convert_cstr("provider_name", provider_name).unwrap_or_else(|| "Provider");
+  let consumer = convert_cstr("consumer_name", consumer_name).unwrap_or("Consumer");
+  let provider = convert_cstr("provider_name", provider_name).unwrap_or("Provider");
   handles::MessagePactHandle::new(consumer, provider)
 }
 
@@ -1143,7 +1194,7 @@ pub extern fn new_message(pact: handles::MessagePactHandle, description: *const
       handles::MessageHandle::new(pact.clone(), inner.messages.len())
     }).unwrap_or_else(|| handles::MessageHandle::new(pact.clone(), 0))
   } else {
-    handles::MessageHandle::new(pact.clone(), 0)
+    handles::MessageHandle::new(pact, 0)
   }
 }
 
@@ -1211,7 +1262,7 @@ pub extern fn message_given_with_param(message: handles::MessageHandle, descript
 /// * `size` - number of bytes in the message to read
 #[no_mangle]
 pub extern fn message_with_contents(message: handles::MessageHandle, content_type: *const c_char, body: *const c_char, size: size_t) {
-  let content_type = convert_cstr("content_type", content_type).unwrap_or_else(|| "text/plain");
+  let content_type = convert_cstr("content_type", content_type).unwrap_or("text/plain");
 
   message.with_message(&|_, inner| {
     let content_type = ContentType::parse(content_type).ok();
@@ -1320,7 +1371,7 @@ pub extern fn with_message_pact_metadata(pact: handles::MessagePactHandle, names
     let name = convert_cstr("name", name).unwrap_or_default();
     let value = convert_cstr("value", value).unwrap_or_default();
 
-    if namespace != "" {
+    if !namespace.is_empty() {
       let mut child = BTreeMap::new();
       child.insert(name.to_string(), value.to_string());
       inner.metadata.insert(namespace.to_string(), child);
